/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.person.apio.internal.architect.form;

import com.liferay.apio.architect.form.Form;
import com.liferay.portal.kernel.util.Validator;

import java.util.Calendar;
import java.util.Date;
import java.util.Optional;

/**
 * Instances of this class represent the values extracted from a person creator
 * form.
 *
 * @author Alejandro Hernández
 * @review
 */
public class PersonCreatorForm {

	/**
	 * Builds a {@code Form} that generates {@code PersonCreatorForm} depending
	 * on the HTTP body.
	 *
	 * @param  formBuilder the {@code Form} builder
	 * @return a person creator form
	 */
	public static Form<PersonCreatorForm> buildForm(
		Form.Builder<PersonCreatorForm> formBuilder) {

		return formBuilder.title(
			__ -> "The person creator form"
		).description(
			__ -> "This form can be used to create a person"
		).constructor(
			PersonCreatorForm::new
		).addOptionalString(
			"gender", PersonCreatorForm::_setGender
		).addOptionalString(
			"alternateName", PersonCreatorForm::_setAlternateName
		).addOptionalDate(
			"birthDate", PersonCreatorForm::_setBirthDate
		).addRequiredString(
			"email", PersonCreatorForm::_setEmail
		).addRequiredString(
			"familyName", PersonCreatorForm::_setFamilyName
		).addRequiredString(
			"givenName", PersonCreatorForm::_setGivenName
		).addOptionalString(
			"jobTitle", PersonCreatorForm::_setJobTitle
		).build();
	}

	/**
	 * Returns the person's alternate name
	 *
	 * @return the person's alternate name
	 * @review
	 */
	public String getAlternateName() {
		return _alternateName;
	}

	/**
	 * Returns the person's birthday day
	 *
	 * @return the person's birthday day
	 * @review
	 */
	public int getBirthdayDay() {
		if (_birthdayDay != null) {
			return _birthdayDay;
		}

		return 1;
	}

	/**
	 * Returns the person's birthday month
	 *
	 * @return the person's birthday month
	 * @review
	 */
	public int getBirthdayMonth() {
		if (_birthdayMonth != null) {
			return _birthdayMonth;
		}

		return 0;
	}

	/**
	 * Returns the person's birthday year
	 *
	 * @return the person's birthday year
	 * @review
	 */
	public int getBirthdayYear() {
		if (_birthdayYear != null) {
			return _birthdayYear;
		}

		return 1970;
	}

	/**
	 * Returns the person's email
	 *
	 * @return the person's email
	 * @review
	 */
	public String getEmail() {
		return _email;
	}

	/**
	 * Returns the person's family name
	 *
	 * @return the person's family name
	 * @review
	 */
	public String getFamilyName() {
		return _familyName;
	}

	/**
	 * Returns the person's given name
	 *
	 * @return the person's given name
	 * @review
	 */
	public String getGivenName() {
		return _givenName;
	}

	/**
	 * Returns the person's job title
	 *
	 * @return the person's job title
	 * @review
	 */
	public String getJobTitle() {
		return Optional.ofNullable(
			_jobTitle
		).orElse(
			""
		);
	}

	/**
	 * Checks if the person is a male
	 *
	 * @return {@code true} if the person is a male; {@code false} otherwise
	 * @review
	 */
	public boolean isMale() {
		return Optional.ofNullable(
			_male
		).orElse(
			true
		);
	}

	/**
	 * Checks if the person has an alternate name
	 *
	 * @return {@code true} if the person has an alternate name; {@code false}
	 *         otherwise
	 * @review
	 */
	public boolean needsAlternateName() {
		return Validator.isNull(_alternateName);
	}

	private void _setAlternateName(String alternateName) {
		_alternateName = alternateName;
	}

	private void _setBirthDate(Date birthDate) {
		Calendar calendar = Calendar.getInstance();

		calendar.setTime(birthDate);

		_birthdayMonth = calendar.get(Calendar.MONTH);
		_birthdayDay = calendar.get(Calendar.DATE);
		_birthdayYear = calendar.get(Calendar.YEAR);
	}

	private void _setEmail(String emailAddress) {
		_email = emailAddress;
	}

	private void _setFamilyName(String lastName) {
		_familyName = lastName;
	}

	private void _setGender(String gender) {
		_male = "male".equals(gender);
	}

	private void _setGivenName(String givenName) {
		_givenName = givenName;
	}

	private void _setJobTitle(String jobTitle) {
		_jobTitle = jobTitle;
	}

	private String _alternateName;
	private Integer _birthdayDay;
	private Integer _birthdayMonth;
	private Integer _birthdayYear;
	private String _email;
	private String _familyName;
	private String _givenName;
	private String _jobTitle;
	private Boolean _male;

}